/*
  See how many points can get within time limit
  Difficulty & points get per thing adjustable via potentiometer
*/

#include <math.h>

// initialize pins
int ledPins[] = {5,7,9,11,12};
int numLeds = 5;
int switchPin = 2;
int potPin = A0;

// track switch presses
int prevState;
int curState;
//int presses = 0;
boolean switchPressed = false;

// various delays
int margin = 100; // delay for just enough time for switch 
                  //   to settle on a value (contact bounce)
//int blinkDur = 100;
int waveDur = 0; // depends on potentiometer

// track time
unsigned long prevTime = 0;
unsigned long curTime = 0;
long remainingTime = 90000;  // 1min30sec
boolean alerted = false;   // 30sec alert

boolean gameOver = false;
int totalScore = 0;
int points = 0; // depends on potentiometer


void setup()
{
  Serial.begin(9600);
  
  pinMode(switchPin, INPUT);  
  prevState = digitalRead(switchPin); // read initial state
    
  for (int i = 0; i < numLeds; i++){
    pinMode(ledPins[i], OUTPUT);
  }                    

  startGame();  
}

void loop()
{    
  if(!gameOver){ 
    // get latest value from potentiometer
    waveDur = map(analogRead(potPin),0,1023,0,200);
    points = int( map(analogRead(potPin),0,1023,7,1) );
    
    wave();
    
    // update display
    updateScore();
    updateTime();
  }
  else{
    checkPress();
    
    if(switchPressed){
      gameOver = false;     // new round
      startGame();
    }
      
    switchPressed = false;  // reset
  }
}

void checkPress()
{
  curState = digitalRead(switchPin);
  
  if(curState != prevState){  // button state has changed
    if (curState == HIGH){    // button pressed down
      
      switchPressed = true;      
      
      //presses++;
      //Serial.print("press # "); Serial.println(presses);  //uncomment for debugging

      delay(margin);
    }
    else                    // button released
      delay(margin);
  }
  
  prevState = curState;
}

void wave()  // includes hit checker
{
  //Serial.println("Should be a wave");    //uncomment for debugging
  
  /* hacky -- allow for higher chance that press of
              switchPin during Wave loop is registered */
       
  for (int i = 0; i < numLeds; i++)
  {
    allOff();   //should be faster than eye can notice
    digitalWrite(ledPins[i], HIGH);
    //Serial.print("led "); Serial.print(i); Serial.println(" is on");  //uncomment for debugging
    
    for(int j = 0; j < 3; j++){  // call checkPress() j-times while pin is HIGH
      delay(waveDur);            // a
      checkPress();              // b; a+b=total delay
        
      if(switchPressed && i == 2){
        totalScore += points;    // add score
        //Serial.println("yay a hit");  //uncomment for debugging
      }
      
      switchPressed = false;     // reset
    } 
  }
 
  for (int i = numLeds - 2; i > 0; i--){
    allOff();
    digitalWrite(ledPins[i], HIGH);
    //Serial.print("led "); Serial.print(i); Serial.println(" is on");  //uncomment for debugging
    
    for(int j = 0; j < 3; j++){
      delay(waveDur); 
      checkPress();
      
      if(switchPressed && i == 2){
        totalScore += points;
        //Serial.println("yay a hit");   //uncomment for debugging
      }
      
      switchPressed = false; 
    } 
  }
}


void updateScore(){
  // Serial.print("score is "); Serial.println(totalScore);
  Serial.print('$'); Serial.print(totalScore); 
  Serial.print(' '); Serial.println('*'); //format for Processing sketch
}

void updateTime()
{
  curTime = millis();
  long change = curTime - prevTime;
  remainingTime -= change;
  prevTime = curTime;
    
  if (remainingTime <= 0){
    remainingTime = 0;
    formatTime(remainingTime);
    endGame();
  }
  else if (remainingTime <= 30000 && !alerted){
    alertAnim();
    alerted = true;
    formatTime(remainingTime);
  }
  else  
    formatTime(remainingTime); // send to Processing
}

void startGame()
{
  startAnim(); 
  prevTime = millis();
}

void endGame()
{  
  
  gameOver = true;
  // Serial.println("game over!");
  Serial.println("gEnd*"); //format for Processing sketch
  
  endAnim();
  
  //print final score
  
  reset();
}
          
void reset()
{
  totalScore = 0;
  remainingTime = 90000;
}

void formatTime(unsigned long t)  //millis
{  
  // Serial.print("unformatted time "); Serial.println(t);   //uncomment for debugging
  if (t < 0)
    t = 0;
    
  // minutes
  int m = floor( t / 60000 );
  t = t % 60000;
  // seconds
  int s = floor( t / 1000 );
  
  //Serial.print("time is ");
  Serial.print('^'); //format for Processing sketch
  Serial.print( m );
  Serial.print( ':' );  
  if (s < 10)
    Serial.print( 0 ); 
  Serial.print( s );
  Serial.println('*'); //format for Processing sketch
   
}


// animation stuff
void allOn()
{
  for (int i = 0; i < numLeds; i++){
    digitalWrite(ledPins[i], HIGH);
  }    
  //Serial.println("should all be on");   //uncomment for debugging
}

void allOff()
{
  for (int i = 0; i < numLeds; i++){
    digitalWrite(ledPins[i], LOW);
  }   
  //Serial.println("should all be off");  //uncomment for debugging
}

void allBlink(int n, int dur)
{
  for (int i = 0; i < n; i++){
    allOn(); delay(dur);
    allOff(); delay(dur);
  }
}

void startAnim()
{
  allOff();
  allBlink(3,500);   // blink 3x
} 

void endAnim()
{
  allOn(); delay(500);
  allOff();
}

void alertAnim()
{
  allBlink(1,300); // blink once
  
  //Serial.println("30sec warning");
  Serial.println("30sec*");
}
